%% scanned target calibration (version 1)

%% image

clear
close all
rgb = imread('Scan_Pic0003.jpg');
%rgb = imread('target.jpg');
img = im2double(rgb2gray(rgb));
sz = size(img);
img = img(400:sz(1),:);
im = imresize(img,0.25);
imshow(im);



%%

h = filters2();
h0 = h(:,:,1);
h1 = h(:,:,2);
h2 = h(:,:,3);
h11 = h(:,:,4);
h22 = h(:,:,5);
h12 = h(:,:,6);

%% edges

a1 = imfilter(im,h1);
a2 = imfilter(im,h2);
gmax = max(max(a1));
gmin = min(min(a1));
a1max = max(gmax,-gmin);
fprintf('a1 max %g min %g\n',gmax,gmin);
gmax = max(max(22));
gmin = min(min(a2));
a2max = max(gmax,-gmin);
fprintf('a2 max %g min %g\n',gmax,gmin);
edge = sqrt(a1.^2+a2.^2);
gmax = max(max(edge));
gmin = min(min(edge));
fprintf('edge max %g min %g\n',gmax,gmin);
imshow(edge>0.5*gmax);

%%


b11 = imfilter(im,h11);
gmax = max(max(b11));
gmin = min(min(b11));
fprintf('b11 max %g min %g\n',gmax,gmin);
b22 = imfilter(im,h22);
gmax = max(max(b11));
gmin = min(min(b11));
fprintf('b22 max %g min %g\n',gmax,gmin);
bs = imfilter(im,h12);
gmax = max(max(bs));
gmin = min(min(bs));
fprintf('bs max %g min %g\n',gmax,gmin);
bsmax = max(gmax,-gmin);

br = b11+b22;
gmax = max(max(br));
gmin = min(min(br));
fprintf('br max %g min %g\n',gmax,gmin);
bc = b11-b22;
gmax = max(max(bc));
gmin = min(min(bc));
fprintf('bc max %g min %g\n',gmax,gmin);

%% corner operator

ac = a1.^2 - a2.^2;
as = 2*a1.*a2;
e = bc.*as-bs.*ac;
gmax = max(max(e));
gmin = min(min(e));
fprintf('corner max %g min %g\n',gmax,gmin);
emax = max(gmax,-gmin)*0.5;
e = abs(e/emax);
d = cat(3,e,e,4*e);
imshow(d)
%bp2(e,1,0.5);

%% basis images

a0 = imresize(im,1/3);
sz = size(a0);
data = zeros(3*sz(1),3*sz(2));
xd = 1:sz(1);
yd = 1:sz(2);
data(xd,yd)=a0;

% first derivatives
data(xd,sz(2)+yd)=bp2(a1,1/3,2);
data(sz(1)+xd,yd)=bp2(a2,1/3,2);

% second derivatives
data(xd,2*sz(2)+yd)=bp2(bc,1/3,2);
%data(2*sz(1),yd)=imresize(b22,1/3);
data(sz(1)+xd,sz(2)+yd)=bp2(br,1/3,2);
data(2*sz(1)+xd,yd)=bp2(bs,1/3,2);

imshow(data);

%% show vertices

dpi = 599.406; % value from ruler2 (with filtering)
dpmm = dpi/25.4;


c1 = a1/sum(sum(h1.^2));
c2 = a2/sum(sum(h2.^2));
c3 = bs/sum(sum(h12.^2));

xcen = -c2./c3;
ycen = -c1./c3;
r = sqrt(xcen.^2+ycen.^2);
mask = r<sqrt(2).*(abs(bs)>0.7*bsmax);
imshow(mask);

%% calculate vertices

sz = size(im);
xloc = ones(sz(1),1)*(1:sz(2));
yloc = (1:sz(1))'*ones(1,sz(2));
idx = find(mask(:)>0);
xcen=xcen(:);
xloc=xloc(:);
xpt = xloc(idx)+xcen(idx);
ycen=ycen(:);
yloc=yloc(:);
ypt = yloc(idx)+ycen(idx);
plot(xpt,ypt,'o');

%% sort vertices

k = 0;
xw = xpt;
yw = ypt;
clear xc yc
while ~isempty(xw)
    test = (xw-xw(1)).^2+(yw-yw(1)).^2;
    [tsort idx] = sort(test);
    xw = xw(idx);
    yw = yw(idx);
    idx = find(tsort<36);
    k = k + 1;
    xc(k) = mean(xw(idx));
    yc(k) = mean(yw(idx));
    idx = find(tsort>16);
%     k
%     idx'
    xw = xw(idx);
    yw = yw(idx);
end
plot(xc,yc,'o');

%% show results (on image)

imshow(im);
hold on
plot(xc,yc,'x','LineWidth',2,'MarkerSize',10);
hold off

%% widths

pts(:,:,1) = reshape(xc,3,3);
pts(:,:,2) = reshape(yc,3,3);
pts = sort(pts,1);
width = 4*diff(pts(:,:,1)')'/dpmm;
ydiff = 4*diff(pts(:,:,2)')'/dpmm;
fprintf('%12s\t %12s\t %12s\t %12s\n','row','width 1','width 2','row angle');
fprintf('%12s\t %12s\t %12s\t %12s\n','index','mm','mm','degrees');
for n=1:3
    c = polyfit(pts(n,:,1),pts(n,:,2),1);
    fprintf('%12g\t %12g\t %12g\t %12g\n',floor(pts(n,1,2)),width(n,1),width(n,2),atan(c(1))*180/pi);
end

%% heights

height = 4*diff(pts(:,:,2))/dpmm;
xdiff = 4*diff(pts(:,:,1))/dpmm;
fprintf('%12s\t %12s\t %12s\t %12s\n','col','height 1','height 2','col angle');
fprintf('%12s\t %12s\t %12s\t %12s\n','index','mm','mm','degrees');
for n=1:3
    c = polyfit(pts(:,n,2),pts(:,n,1),1);
    fprintf('%12g\t %12g\t %12g\t %12g\n',floor(pts(1,n,1)),height(1,n),height(2,n),atan(c(1))*180/pi);
end











